package jemu.core.device.tape;

import jemu.core.cpu.Z80;
import java.io.IOException;
import java.util.Vector;

//
//            CDTPlayerStatus

public class CPCCDTFilePlayer
{

    private Z80 z80;
    private boolean ivPlaying;
    private byte ivTzxFile[];
    private int ivEventStack[];
    private int ivEventStackPointer;
    private long ivCycleNextEvent;
    private int ivBlockPos;
    private int ivBlockLen;
    private int ivBlockType;
    private byte ivStreamerBuffer[];
    private int ivStreamerBitPos;
    private int ivStreamerBitPosEnd;
    private boolean ivLevel;
    private static final int OP_STACKEND = 0;
    private static final int OP_NOP = 1;
    private static final int OP_PAUSE = 2;
    private static final int OP_LOW = 3;
    private static final int OP_SWITCH = 4;
    private static final int OP_PULSES = 5;
    private static final int OP_BITSTREAM = 6;
    private static final int OP_LOOP = 7;
    private static final int OP_SYNCHRONIZE = 8;
    private static final int OP_NEXTBLOCK = 9;

    public CPCCDTFilePlayer()
    {
        resetEventStack();
    }

    public CPCCDTFilePlayer(int port, byte abyte0[])
    {
        setTzxFileBuffer(abyte0, 0);
    }

    public CPCCDTFilePlayer(String s)
        throws IOException
    {
      //  this(zxexpansionport, fileloader.readFile(s, -1));
    }

    private void advanceToCycle(long l)
    {
        while(ivCycleNextEvent < l) 
        {
            advanceToNextEvent();
        }
    }
 public int readIO(int i)
    {
        if(i!=0)
        {
            if(ivPlaying)
            {
                try
                {
                    advanceToCycle(1000000);
                }
                catch(Throwable throwable)
                {
                    System.out.println("Error in TZX file.");
                    throwable.printStackTrace();

                 ivPlaying = false;
                }
            }
            if(ivLevel)
            {
                return 0x080;	//j & 0xbf;
            } else
            {
                return 0x000;
            }
        } else
        {
            return 0x000;
        }
    }

    private void advanceToNextEvent()
    {
        try
        {
            int i = pop();
            switch(i)
            {
            case 0: // '\0'
                push(0);
                ivPlaying = false;
                ivCycleNextEvent = 0x7fffffffffffffffL;
                break;

            case 1: // '\001'
                break;

            case 2: // '\002'
                ivCycleNextEvent += pop();
                break;

            case 3: // '\003'
                ivLevel = false;
                // fall through

            case 4: // '\004'
                ivLevel = !ivLevel;
                break;

            case 5: // '\005'
                int j = pop();
                int k = pop();
                if(j > 0)
                {
                    push(k);
                    push(--j);
                    push(5);
                    ivCycleNextEvent += k;
                    push(4);
                }
                break;

            case 6: // '\006'
                int l = pop();
                int i1 = pop();
                if(!streamerIsEmpty())
                {
                    push(i1);
                    push(l);
                    push(6);
                    push(streamerReadNext() ? l : i1);
                    push(2);
                    push(5);
                }
                break;

            case 7: // '\007'
                int j1 = pop();
                int k1 = pop();
                if(j1 > 0)
                {
                    ivBlockPos = k1;
                    ivBlockLen = 0;
                    push(k1);
                    push(--j1);
                    push(7);
                    push(9);
                }
                break;

            case 8: // '\b'
                ivCycleNextEvent = z80.getCyclesPerSecond();
                break;

            case 9: // '\t'
                push(9);
                if(gotoNextBlock())
                {
                    pop();
                }
                break;

            default:
                throw new RuntimeException("unknown function: " + i);
            }
        }
        catch(Exception exception)
        {
            System.out.println("Event stack problem:");
            exception.printStackTrace();
            ivCycleNextEvent = 0x7fffffffL;
            ivPlaying = false;
        }
    }


    public CDTPlayerStatus getStatus()
    {
        throw new RuntimeException("Not implemented");
    }

    private boolean gotoNextBlock()
    {
        ivBlockPos += ivBlockLen + 1;
        if(ivBlockPos <= ivTzxFile.length)
        {
            ivBlockType = ivTzxFile[ivBlockPos - 1];
            switch(ivBlockType)
            {
            case 16: // '\020'
                parse10();
                break;

            case 17: // '\021'
                parse11();
                break;

            case 18: // '\022'
                parse12();
                break;

            case 19: // '\023'
                parse13();
                break;

            case 20: // '\024'
                parse14();
                break;

            case 21: // '\025'
                parse15();
                break;

            case 22: // '\026'
                parse16();
                break;

            case 23: // '\027'
                parse17();
                break;

            case 32: // ' '
                parse20();
                break;

            case 33: // '!'
                parse21();
                break;

            case 34: // '"'
                parse22();
                break;

            case 35: // '#'
                parse23();
                break;

            case 36: // '$'
                parse24();
                break;

            case 37: // '%'
                parse25();
                break;

            case 38: // '&'
                parse26();
                break;

            case 39: // '\''
                parse27();
                break;

            case 40: // '('
                parse28();
                break;

            case 42: // '*'
                parse2a();
                break;

            case 48: // '0'
                parse30();
                break;

            case 49: // '1'
                parse31();
                break;

            case 50: // '2'
                parse32();
                break;

            case 51: // '3'
                parse33();
                break;

            case 52: // '4'
                parse34();
                break;

            case 53: // '5'
                parse35();
                break;

            case 64: // '@'
                parse40();
                break;

            case 90: // 'Z'
                parse5a();
                break;

            default:
                System.out.println("Unknown TZX blocktype: " + ivBlockType);
                return gotoNextBlock();
            }
            return false;
        } else
        {
            return true;
        }
    }

    private void ignore()
    {
    }

    public static boolean isProgram(byte abyte0[])
    {
        for(int i = 10; i < abyte0.length - 19; i++)
        {
            if(abyte0[i + 0] == 16 && abyte0[i + 3] == 19 && abyte0[i + 4] == 0 && abyte0[i + 5] == 0)
            {
                return abyte0[i + 6] != 3;
            }
        }

        return true;
    }

    public static boolean isTzxFile(byte abyte0[])
    {
        throw new RuntimeException("Not implemented");
    }

    public Object[] list()
    {
        Vector vector = new Vector();
        while(ivBlockPos < ivTzxFile.length) 
        {
            gotoNextBlock();
            vector.addElement("ID: " + Array.b2x(new byte[] {
                (byte)ivBlockType
            }) + ", pos: " + ivBlockPos + ", len: " + ivBlockLen);
        }
        return vector.toArray();
    }

    public static void main(String args[])
    {
    }

    private void parse10()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos + 2, 2) + 4;
        streamerLoad(ivTzxFile, ivBlockPos + 4, (ivBlockLen - 4) * 8);
        pushPause(toIntLsbf(ivTzxFile, ivBlockPos, 2));
        pushBitStream(855, 1710);
        pushPulses(735, 1);
        pushPulses(667, 1);
        pushPulses(2168, ivTzxFile[ivBlockPos + 4] < 0 ? 3220 : 8064);
    }

    private void parse11()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos + 15, 3) + 18;
        streamerLoad(ivTzxFile, ivBlockPos + 18, ((ivBlockLen - 18) * 8 - 8) + toIntLsbf(ivTzxFile, ivBlockPos + 12, 1));
        pushPause(toIntLsbf(ivTzxFile, ivBlockPos + 13, 2));
        pushBitStream(toIntLsbf(ivTzxFile, ivBlockPos + 6, 2), toIntLsbf(ivTzxFile, ivBlockPos + 8, 2));
        pushPulses(toIntLsbf(ivTzxFile, ivBlockPos + 4, 2), 1);
        pushPulses(toIntLsbf(ivTzxFile, ivBlockPos + 2, 2), 1);
        pushPulses(toIntLsbf(ivTzxFile, ivBlockPos + 0, 2), toIntLsbf(ivTzxFile, ivBlockPos + 10, 2));
    }

    private void parse12()
    {
        ivBlockLen = 4;
        pushPulses(toIntLsbf(ivTzxFile, ivBlockPos + 0, 2), toIntLsbf(ivTzxFile, ivBlockPos + 2, 2));
    }

    private void parse13()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 1) * 2 + 1;
        for(int i = (ivBlockPos + ivBlockLen) - 2; i >= ivBlockPos + 1; i -= 2)
        {
            pushPulses(toIntLsbf(ivTzxFile, i, 2), 1);
        }

    }

    private void parse14()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos + 7, 3) + 10;
        streamerLoad(ivTzxFile, ivBlockPos + 10, ((ivBlockLen - 10) * 8 - 8) + toIntLsbf(ivTzxFile, ivBlockPos + 4, 1));
        pushPause(toIntLsbf(ivTzxFile, ivBlockPos + 5, 2));
        pushBitStream(toIntLsbf(ivTzxFile, ivBlockPos + 0, 2), toIntLsbf(ivTzxFile, ivBlockPos + 2, 2));
    }

    private void parse15()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos + 5, 3) + 8;
        ignore();
    }

    private void parse16()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 4);
        ignore();
    }

    private void parse17()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 4);
        ignore();
    }

    private void parse20()
    {
        ivBlockLen = 2;
    }

    private void parse21()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 1) + 1;
        ignore();
    }

    private void parse22()
    {
        ivBlockLen = 0;
        ignore();
    }

    private void parse23()
    {
        ivBlockLen = 2;
        ignore();
    }

    private void parse24()
    {
        ivBlockLen = 2;
        push(ivBlockPos + ivBlockLen);
        push(toIntLsbf(ivTzxFile, ivBlockPos, 2));
        push(7);
    }

    private void parse25()
    {
        ivBlockLen = 0;
        pop();
    }

    private void parse26()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 2) * 2 + 2;
        ignore();
    }

    private void parse27()
    {
        ivBlockLen = 0;
        ignore();
    }

    private void parse28()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 2) * 2 + 2;
        ignore();
    }

    private void parse2a()
    {
        ivBlockLen = 4;
        ignore();
    }

    private void parse30()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 1) + 1;
        ignore();
    }

    private void parse31()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos + 1, 1) + 2;
        ignore();
    }

    private void parse32()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 2) + 2;
        ignore();
    }

    private void parse33()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 1) * 3 + 1;
        ignore();
    }

    private void parse34()
    {
        ivBlockLen = 8;
        ignore();
    }

    private void parse35()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 4) + 14;
        ignore();
    }

    private void parse40()
    {
        ivBlockLen = toIntLsbf(ivTzxFile, ivBlockPos, 3) + 4;
        ignore();
    }

    private void parse5a()
    {
        ivBlockLen = 9;
        ignore();
    }

    private int pop()
    {
        return ivEventStack[ivEventStackPointer--];
    }

    private void push(int i)
    {
        ivEventStack[++ivEventStackPointer] = i;
    }

    private void pushBitStream(int i, int j)
    {
        push(i);
        push(j);
        push(6);
    }

    private void pushPause(int i)
    {
        if(i > 0)
        {
            push((i - 1) * 3500);
            push(2);
            push(3);
            push(3500);
            push(2);
        }
    }

    private void pushPulses(int i, int j)
    {
        push(i);
        push(j);
        push(5);
    }

    public int readIO(int i, int j)
    {
        if((i & 1) == 0)
        {
            if(ivPlaying)
            {
                try
                {
                    advanceToCycle(z80.getCyclesPerSecond());
                }
                catch(Throwable throwable)
                {
                    System.out.println("Error in TZX file.");
                    throwable.printStackTrace();
                    ivPlaying = false;
                }
            }
            if(ivLevel)
            {
                return j & 0xbf;
            } else
            {
                return j;
            }
        } else
        {
            return j;
        }
    }

    public void readOpcode1(int i)
    {
    }

    public int readOpcode2(int i, int j)
    {
        return j;
    }

    private void resetEventStack()
    {
        ivEventStack = new int[1000];
        ivEventStackPointer = 0;
    }

    public void setStatus(CDTPlayerStatus tzxfileplayerstatus)
    {
        setTzxFileBuffer(tzxfileplayerstatus.getCDTFileBytes(), 0);
    }

    public void setTzxFileBuffer(byte abyte0[], int i)
    {
        ivTzxFile = abyte0;
        ivBlockPos = 0;
        ivBlockLen = 10;
        ivCycleNextEvent = 0L;
        resetEventStack();
        push(9);
        pushPause(i);
        push(8);
        ivPlaying = true;
    }

    private boolean streamerIsEmpty()
    {
        return ivStreamerBitPos >= ivStreamerBitPosEnd;
    }

    private void streamerLoad(byte abyte0[], int i, int j)
    {
        ivStreamerBuffer = abyte0;
        ivStreamerBitPos = i << 3;
        ivStreamerBitPosEnd = ivStreamerBitPos + j;
    }

    private boolean streamerReadNext()
    {
        boolean flag = (ivStreamerBuffer[ivStreamerBitPos >> 3] << (ivStreamerBitPos & 7) & 0x80) != 0;
        ivStreamerBitPos++;
        return flag;
    }

    public void terminate()
    {
    }

    private static int toIntLsbf(byte abyte0[], int i, int j)
    {
        return Array.toIntLsbf(abyte0, i, j);
    }

    public void writeIO(int i, int j)
    {
    }
}
